@@ -1672,9 +1672,10 @@ class profile
 
     // Validate a resource buffer per profile.json validate json node
     // "validate": {
-    //   "size": 0,   // unused for now
-    //   "offset": 0, // unused for now
-    //   "file": "gold.bin"
+    //   "file": "gold.bin", // path to file
+    //   "skip": 0,          // skip fist bytes of file (optional)
+    //   "begin": 0,         // bo offset to start validating at (optional)
+    //   "end": bo.size()    // bo offset to start validating at (optional)
     //  }
     void
     validate_buffer(xrt::bo& bo, const validate_node& node, const artifacts::repo* repo)
@@ -1697,19 +1698,27 @@ class profile
         golden_data = std::string_view{golden_data.data() + skip, golden_data.size() - skip};
       }
 
-      // here we could extract offset and size of region to validate
-      
       bo.sync(XCL_BO_SYNC_BO_FROM_DEVICE);
-      auto bo_data = bo.map<char*>();
+
       if (bo.size() != golden_data.size())
         throw validation_error("Size mismatch during validation");
 
-      if (std::equal(golden_data.data(), golden_data.data() + golden_data.size(), bo_data))
+      // Optional range of bo to validate
+      auto bo_begin = node.value<size_t>("begin", 0);
+      auto bo_end = node.value<size_t>("end", bo.size());
+      if (bo_begin > bo_end || bo_end > bo.size())
+        throw profile_error("bad validate begin/end values: " + std::to_string(bo_begin) + "/" + std::to_string(bo_end));
+
+      auto bo_data = bo.map<char*>();
+      auto bo_range_bytes = bo_end - bo_begin;
+
+      XRT_DEBUGF("profile::bindings::validate_buffer() validating bo range [%d,%d[\n", bo_begin, bo_end);
+      if (std::equal(bo_data + bo_begin, bo_data + bo_end, golden_data.data()))
         return;
 
       // Error
-      for (uint64_t i = 0; i < golden_data.size(); ++i) {
-        if (golden_data[i] != bo_data[i])
+      for (uint64_t i = 0; i < bo_range_bytes; ++i) {
+        if (golden_data[i] != bo_data[bo_begin + i])
           throw validation_error
             ("gold[" + std::to_string(i) + "] = " + std::to_string(golden_data[i])
              + " does not match bo value in bo " + std::to_string(bo_data[i]));
@@ -1719,8 +1728,10 @@ class profile
     // init_buffer_file() - Initialize bo from a content of a file
     //
     // "init": {
-    //   "file": "path", // path to file
-    //   "skip": bytes,  // skip fist bytes of file (optional)
+    //   "file": "path",  // path to file
+    //   "skip": bytes,   // skip fist bytes of file (optional)
+    //   "begin": 0,      // offset to start writing at (optional)
+    //   "end": bo.size() // offset to end writing at (optional)
     // }
     //
     // This function fills all the bytes of the buffer with data
@@ -1735,10 +1746,15 @@ class profile
     {
       auto file = node.at("file").get<std::string>();
       auto skip = node.value<size_t>("skip", 0);
+      auto bo_begin = node.value<size_t>("begin", 0);
+      auto bo_end = node.value<size_t>("end", bo.size());
       auto data = m_repo->get(file);
       if (skip > data.size())
         throw profile_error("bad skip value: " + std::to_string(skip));
 
+      if (bo_begin > bo_end || bo_end > bo.size())
+        throw profile_error("bad init begin/end values: " + std::to_string(bo_begin) + "/" + std::to_string(bo_end));
+
       // Adjust the view, skipping skip bytes, then copy to bo
       data = std::string_view{data.data() + skip, data.size() - skip};
 
@@ -1747,25 +1763,32 @@ class profile
       if (!bo)
         bo = xrt::ext::bo{m_device, data.size()};
 
-      // Copy bytes from file to bo, wrap around file if needed to
-      // fill the bo with data from file.   Copy at offset into
-      // bo based on number of bytes left to copy.  Compute file
-      // data range to copy to bo.
       auto bo_data = bo.map<char*>();
-      auto bytes = bo.size(); // must fill all bytes of bo
-      XRT_DEBUGF("profile::bindings::init_buffer_file() copying (%d) bytes from file (%s)\n",
-                 bytes, file.c_str());
+
+      // Pad the bo with 0s outside the [bo_begin, bo_end[ range
+      std::fill(bo_data, bo_data + bo_begin, char(0));
+      std::fill(bo_data + bo_end, bo_data + bo.size(), char(0));
+
+      // Copy bytes from file to bo starting at optional begin offset.
+      // Wrap around file if needed to fill the bo with data from
+      // file.  Copy at offset into bo based on number of bytes left
+      // to copy.
+      bo_data += bo_begin; // past optional begin
+      auto bo_range_bytes = bo_end - bo_begin;  // default bo.size()
+
+      // Must fill all bytes of bo in [begin, end[ range
+      auto bytes = bo_range_bytes;
 
       // This loop wraps around the source data if necessary in order
-      // to fill all bytes of the bo.  The loop adjusts for iteration.
+      // to fill all bytes of the bo range.  The loop adjusts for iteration.
       while (bytes) {
-        auto bo_offset = bo.size() - bytes;
-        auto beg = ((iteration * bo.size()) + (bo_offset)) % data.size();
+        auto bo_offset = bo_range_bytes - bytes; // offset within bo_data
+        auto beg = ((iteration * bo_range_bytes) + (bo_offset)) % data.size();
         auto end = std::min<size_t>(beg + bytes, data.size());
         bytes -= end - beg;
 
-        XRT_DEBUGF("profile::bindings::init_buffer_file() (itr,beg,end,bytes)=(%d,%d,%d,%d)\n",
-                   iteration, beg, end, bytes);
+        XRT_DEBUGF("profile::bindings::init_buffer_file() (itr,beg,end,offset)=(%d,%d,%d,%d)\n",
+                   iteration, beg, end, bo_offset);
     
         std::copy(data.data() + beg, data.data() + end, bo_data + bo_offset);
       }
@@ -1785,10 +1808,10 @@ class profile
       auto bo_data = bo.map<uint8_t*>();
       auto stride = node.at("stride").get<size_t>();
       auto value = node.at("value").get<uint64_t>();
-      auto begin = node.value("begin", 0);
-      auto end = node.value("end", bo.size());
+      auto bo_begin = node.value("begin", 0);
+      auto bo_end = node.value("end", bo.size());
       arg_range<uint8_t> vr{&value, sizeof(value)};
-      for (size_t offset = begin; offset < end; offset += stride) 
+      for (size_t offset = bo_begin; offset < bo_end; offset += stride) 
         std::copy_n(vr.begin(), std::min<size_t>(bo.size() - offset, vr.size()), bo_data + offset);
     }
 
@@ -2045,6 +2068,7 @@ class profile
     std::string m_name;
     mode m_mode = mode::none;
     size_t m_depth = 1;
+    size_t m_recipe_runs = 1;  // legacy mode throughput calculation
     executor m_executor;
     size_t m_iterations = 1;
     iteration_node m_iteration;
@@ -2136,9 +2160,10 @@ class profile
   public:
     execution(profile* pr, recipe* rr, const json& j, bool legacy = false)
       : m_profile(pr)
-      , m_name(j.value("name", "default"))
+      , m_name(j.value("name", j.value("mode", "default")))
       , m_mode(to_mode(j.value("mode", "default")))
       , m_depth(get_depth(m_mode, j))
+      , m_recipe_runs(rr->num_runs())
       , m_executor{m_profile, rr, m_depth}
       , m_iterations(j.value("iterations", 1))
       , m_iteration(get_iteration_node(m_mode, j))
@@ -2165,9 +2190,15 @@ class profile
         m_profile->validate();
 
       // NOLINTBEGIN
+      // In legacy mode, the number of recipe::runs is used for
+      // throughput and latency calculatons.  In non-legacy mode, the
+      // recipe runs is always considered as one runlist and
+      // calculations are based on the how many times (depth) the
+      // runlist is duplicated.
+      auto depth = m_legacy ? m_recipe_runs : m_depth;
       auto elapsed = time_ns / 1000;
-      auto latency = time_ns / (1000 * m_iterations * m_depth);
-      auto throughput = (1000000000 * m_iterations * m_depth) / time_ns;
+      auto latency = time_ns / (1000 * m_iterations * depth);
+      auto throughput = (1000000000 * m_iterations * depth) / time_ns;
       // NOLINTEND
 
       m_report["cpu"]["elapsed"] = elapsed;
@@ -2606,7 +2637,7 @@ wait()
 
 std::string
 runner::
-get_report()
+get_report() const
 {
   return handle->get_report();
 }
