@@ -18,8 +18,9 @@
 #define core_common_unistd_h_
 
 #ifndef _WIN32
-# include <unistd.h>
+#include <unistd.h>
 #else
+#include <shlobj.h>
 #endif
 
 namespace xrt_core {
@@ -34,5 +35,16 @@ getpagesize()
 #endif
 }
 
+inline bool
+is_user_privileged()
+{
+#ifndef _WIN32
+  return (getuid() == 0) || (geteuid() == 0);
+#else
+  return IsUserAnAdmin();
+#endif
+}
+
+
 } // xrt_core
 #endif
