@@ -25,8 +25,12 @@
 #define QSPI_CMD_PAGE_PROGRAM			0x02
 /* Random read command */
 #define QSPI_CMD_RANDOM_READ			0x03
-/* Status read command */
+/* Status Reg read command */
 #define QSPI_CMD_STATUSREG_READ			0x05
+/* Config Reg read command */
+#define QSPI_CMD_CONFIGREG_READ			0x15
+/* Security Reg read command */
+#define QSPI_CMD_SECURREG_READ			0x2B
 /* Enable flash write */
 #define QSPI_CMD_WRITE_ENABLE			0x06
 /* 4KB Subsector Erase command */
@@ -71,6 +75,8 @@
 #define QSPI_CMD_SECTOR_ERASE			0xD8
 /* Quad IO Fast Read */
 #define QSPI_CMD_QUAD_IO_READ			0xEB
+/* Global block unlock */
+#define QSPI_CMD_GBULK				0x98
 
 #define	FLASH_ERR(flash, fmt, arg...)	\
 	xocl_err(&flash->pdev->dev, fmt "\n", ##arg)
@@ -124,7 +130,8 @@
 
 #define	MAX_NUM_OF_SLAVES	2
 #define	SLAVE_NONE		(-1)
-#define SLAVE_SELECT_NONE	((1 << MAX_NUM_OF_SLAVES) -1)
+#define	SLAVE_REG_MASK		((1 << MAX_NUM_OF_SLAVES) -1)
+#define SLAVE_SELECT_NONE	SLAVE_REG_MASK
 
 /*
  * We support erasing flash memory at three page unit. Page read-modify-write
@@ -142,6 +149,8 @@ static inline size_t FLASH_PAGE_ROUNDUP(loff_t offset)
 		return round_up(offset, FLASH_PAGE_SIZE);
 	return offset + FLASH_PAGE_SIZE;
 }
+struct xocl_flash;
+static int macronix_configure(struct xocl_flash *flash);
 
 /*
  * Wait for condition to be true for at most 1 second.
@@ -187,6 +196,23 @@ static size_t micron_code2sectors(u8 code)
 	return max_sectors;
 }
 
+/*
+ * QSPI IP on some of the old shell/golden does not support
+ * QSPI_CMD_EXT_QUAD_WRITE, but driver does not know if IP is new or old.
+ * Since we only use micron chip on old boards, we'll just use different
+ * write cmd for micron and macronix chip.
+ */
+
+static u8 micron_write_cmd(void)
+{
+	return QSPI_CMD_QUAD_WRITE;
+}
+
+static u8 macronix_write_cmd(void)
+{
+	return QSPI_CMD_EXT_QUAD_WRITE;
+}
+
 static size_t macronix_code2sectors(u8 code)
 {
 	if (code < 0x38 || code > 0x3c)
@@ -194,14 +220,9 @@ static size_t macronix_code2sectors(u8 code)
 	return (1 << (code - 0x38));
 }
 
-static u8 macronix_write_cmd(void)
+static int micron_configure(struct xocl_flash *flash)
 {
-	return QSPI_CMD_PAGE_PROGRAM;
-}
-
-static u8 micron_write_cmd(void)
-{
-	return QSPI_CMD_QUAD_WRITE;
+	return 0;
 }
 
 /*
@@ -212,9 +233,16 @@ static struct qspi_flash_vendor {
 	const char *vendor_name;
 	size_t (*code2sectors)(u8 code);
 	u8 (*write_cmd)(void);
+	int (*configure)(struct xocl_flash *flash);
 } vendors[] = {
-	{ 0x20, "micron", micron_code2sectors, micron_write_cmd },
-	{ 0xc2, "macronix", macronix_code2sectors, macronix_write_cmd },
+	{
+		0x20, "micron", micron_code2sectors,
+		micron_write_cmd, micron_configure
+	},
+	{
+		0xc2, "macronix", macronix_code2sectors,
+		macronix_write_cmd, macronix_configure
+	},
 };
 
 struct qspi_flash_addr {
@@ -251,6 +279,7 @@ struct xocl_flash {
 	bool busy;
 	bool io_debug;
 	size_t flash_size;
+	size_t num_slaves;
 	u8 *io_buf;
 
 	/* For now, only support QSPI */
@@ -312,6 +341,25 @@ static inline void flash_set_ctrl(struct xocl_flash *flash, u32 ctrl)
 	flash_reg_wr(flash, &flash->qspi_regs->qspi_ctrl, ctrl);
 }
 
+static size_t flash_detect_slaves(struct xocl_flash *flash) 
+{
+	u32 slave_reg;
+	size_t num_slaves;
+
+	/* Clear the slave register to make sure we read correct slave value. */
+	flash_reg_wr(flash, &flash->qspi_regs->qspi_slave, 0xFFFFFFFF);
+	slave_reg = flash_reg_rd(flash, &flash->qspi_regs->qspi_slave);
+
+	if(slave_reg == 0x1)
+		num_slaves = 1;
+	else if(slave_reg == 0x3)
+		num_slaves = 2;
+	else
+		num_slaves = 0;
+
+	return num_slaves;
+}
+
 static inline void flash_activate_slave(struct xocl_flash *flash, int index)
 {
 	u32 slave_reg;
@@ -539,7 +587,54 @@ static bool flash_is_ready(struct xocl_flash *flash)
 	return true;
 }
 
-static int flash_get_ID(struct xocl_flash *flash)
+static int flash_enable_write(struct xocl_flash *flash)
+{
+	u8 cmd = QSPI_CMD_WRITE_ENABLE;
+	int ret = flash_transaction(flash, &cmd, 1, false);
+
+	if (ret)
+		FLASH_ERR(flash, "Failed to enable flash write: %d", ret);
+	return ret;
+}
+
+static bool flash_wait_until_ready(struct xocl_flash *flash)
+{
+	if (FLASH_BUSY_WAIT(flash_is_ready(flash))) {
+		FLASH_ERR(flash, "QSPI flash device is not ready");
+		return false;
+	}
+	return true;
+}
+
+static int macronix_configure(struct xocl_flash *flash)
+{
+	int ret;
+	u8 cmd[3];
+
+	FLASH_INFO(flash, "Configuring registers for Macronix");
+
+	//Configure status register (Quad enable, default drive strength)
+	if (!flash_wait_until_ready(flash))
+		return -EINVAL;
+
+	flash_enable_write(flash);
+	cmd[0] = QSPI_CMD_STATUSREG_WRITE;
+	cmd[1] = 0x40;
+	cmd[2] = 0x07;
+	ret = flash_transaction(flash, cmd, 3, false);
+	if (ret)
+		return ret;
+
+	//Set gang block unlock
+	if (!flash_wait_until_ready(flash))
+		return -EINVAL;
+
+	flash_enable_write(flash);
+	cmd[0] = QSPI_CMD_GBULK;
+	return flash_transaction(flash, cmd, 1, false);
+}
+
+static int flash_get_info(struct xocl_flash *flash)
 {
 	int i;
 	struct qspi_flash_vendor *vendor = NULL;
@@ -583,16 +678,6 @@ static int flash_get_ID(struct xocl_flash *flash)
 	return 0;
 }
 
-static int flash_enable_write(struct xocl_flash *flash)
-{
-	u8 cmd = QSPI_CMD_WRITE_ENABLE;
-	int ret = flash_transaction(flash, &cmd, 1, false);
-
-	if (ret)
-		FLASH_ERR(flash, "Failed to enable flash write: %d", ret);
-	return ret;
-}
-
 static int flash_set_sector(struct xocl_flash *flash, u8 sector)
 {
 	int ret = 0;
@@ -660,21 +745,14 @@ flash_setup_io_cmd_header(struct xocl_flash *flash,
 		flash->io_buf[3] = faddr->addr_lo;
 		*header_len = 4;
 	}
-	return ret;
-}
 
-static bool flash_wait_until_ready(struct xocl_flash *flash)
-{
-	if (FLASH_BUSY_WAIT(flash_is_ready(flash))) {
-		FLASH_ERR(flash, "QSPI flash device is not ready");
-		return false;
-	}
-	return true;
+	return ret;
 }
 
 static int qspi_probe(struct xocl_flash *flash)
 {
 	int ret;
+	size_t i;
 
 	/* Probing on first flash only. */
 	flash->qspi_curr_slave = 0;
@@ -690,11 +768,29 @@ static int qspi_probe(struct xocl_flash *flash)
 	if (!flash_wait_until_ready(flash))
 		return -EINVAL;
 
-	/* Update flash vendor. */
-	ret = flash_get_ID(flash);
+	flash->num_slaves = flash_detect_slaves(flash);
+	if(flash->num_slaves == 0)
+		return -EINVAL;
+	FLASH_INFO(flash, "Number of slave chips is: %ld", flash->num_slaves);
+
+	/*
+	 * Get flash info only from first chip assuming that all chips
+	 * are identical. If not, we have a much bigger problem to solve.
+	 */
+	flash->qspi_curr_slave = 0;
+	ret = flash_get_info(flash);
 	if (ret)
 		return ret;
 
+	/* Configure all chips, if necessary. */
+	for (i = 0; i < flash->num_slaves; ++i) {
+		flash->qspi_curr_slave = i;
+		ret = flash->vendor->configure(flash);
+		if (ret)
+			return ret;
+	}
+
+	flash->qspi_curr_slave = 0;
 	flash->qspi_curr_sector = 0xff;
 
 	return 0;
@@ -924,9 +1020,13 @@ flash_do_read(struct xocl_flash *flash, char *kbuf, size_t n, loff_t off)
 	mutex_lock(&flash->io_lock);
 
 	flash_offset2faddr(off, &faddr);
+	if (faddr.slave >= flash->num_slaves) {
+		FLASH_ERR(flash, "Can't read: out of slave boundary");
+		ret = -ENOSPC;
+	}
 	flash->qspi_curr_slave = faddr.slave;
 
-	if (!flash_wait_until_ready(flash))
+	if (ret == 0 && !flash_wait_until_ready(flash))
 		ret = -EINVAL;
 
 	while (ret == 0 && cnt < n) {
@@ -1099,9 +1199,13 @@ flash_write(struct file *file, const char __user *buf, size_t n, loff_t *off)
 	mutex_lock(&flash->io_lock);
 
 	flash_offset2faddr(*off, &faddr);
+	if (faddr.slave >= flash->num_slaves) {
+		FLASH_ERR(flash, "Can't write: out of slave boundary");
+		ret = -ENOSPC;
+	}
 	flash->qspi_curr_slave = faddr.slave;
 
-	if (!flash_wait_until_ready(flash))
+	if (ret == 0 && !flash_wait_until_ready(flash))
 		ret = -EINVAL;
 
 	while (ret == 0 && cnt < n) {
@@ -1293,7 +1397,7 @@ static void sysfs_destroy_flash(struct xocl_flash *flash)
 		sysfs_remove_group(&flash->pdev->dev.kobj, &flash_attr_group);
 }
 
-static int flash_remove(struct platform_device *pdev)
+static int __flash_remove(struct platform_device *pdev)
 {
 	struct xocl_flash *flash;
 	void *hdl;
@@ -1306,6 +1410,7 @@ static int flash_remove(struct platform_device *pdev)
 	platform_set_drvdata(pdev, NULL);
 
 	sysfs_destroy_flash(flash);
+
 	if (flash->io_buf)
 		vfree(flash->io_buf);
 
@@ -1317,6 +1422,15 @@ static int flash_remove(struct platform_device *pdev)
 	return 0;
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 11, 0)
+static void flash_remove(struct platform_device *pdev)
+{
+	__flash_remove(pdev);
+}
+#else
+#define flash_remove __flash_remove
+#endif
+
 static int flash_probe(struct platform_device *pdev)
 {
 	struct xocl_flash *flash;
