@@ -1,18 +1,6 @@
-/*
- * Copyright (C) 2019 Xilinx, Inc
- *
- * Licensed under the Apache License, Version 2.0 (the "License"). You may
- * not use this file except in compliance with the License. A copy of the
- * License is located at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
- * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
- * License for the specific language governing permissions and limitations
- * under the License.
- */
+// SPDX-License-Identifier: Apache-2.0
+// Copyright (C) 2019 Xilinx, Inc
+// Copyright (C) 2022 Advanced Micro Devices, Inc. - All rights reserved
 
 #include <errno.h>
 
@@ -36,10 +24,13 @@
 #include <exception>
 #include <future>
 #include <uuid/uuid.h>
-#include "xclbin.h"
+#include "xrt/detail/xclbin.h"
 #include "aws_dev.h"
 
 static std::map<std::string, size_t>index_map;
+#ifndef INTERNAL_TESTING_FOR_AWS
+static std::thread rescan_thread[16];
+#endif
 /*
  * Functions each plugin needs to provide
  */
@@ -55,7 +46,7 @@ void fini(void *mpc_cookie);
 int init(mpd_plugin_callbacks *cbs)
 {
     int ret = 1;
-    auto total = pcidev::get_dev_total();
+    auto total = xrt_core::pci::get_dev_total();
     if (total == 0) {
         syslog(LOG_INFO, "aws: no device found");
         return ret;
@@ -451,15 +442,85 @@ int awsReadP2pBarAddr(size_t index, const xcl_mailbox_p2p_bar_addr *addr, int *r
     return ret;
 }
 
+#ifndef INTERNAL_TESTING_FOR_AWS
+static void awsPciRescan(int index)
+{
+    std::string sysfs_name = xrt_core::pci::get_dev(index, true)->m_sysfs_name;
+    int mBoardNumber = index_map[sysfs_name];
+    std::this_thread::sleep_for(std::chrono::seconds(1));
+    int dev_offline = -1;
+    std::string err;
+    xclDeviceHandle handle;
+    uint64_t hostmem_size;
+    int ret;
+    
+    // removal & rescan will make the host mem configed lost. so if there is host mem config
+    // save the info and reconfig it after the removal & rescan
+    xrt_core::pci::get_dev(index)->sysfs_get("", "host_mem_size", err, hostmem_size, static_cast<uint64_t>(0));
+    if (hostmem_size)
+        std::cout << "host mem config information saved: " << hostmem_size << std::endl;
+    
+    fpga_pci_rescan_slot_app_pfs(mBoardNumber);
+    while (dev_offline) {
+        std::this_thread::sleep_for(std::chrono::milliseconds(500));
+	xrt_core::pci::get_dev(index)->sysfs_get<int>("", "dev_offline", err, dev_offline, -1);
+    }
+    if (!hostmem_size) {
+        //no need to reconfig host mem, just tell the user xclbin load ioctl can be re-issued now
+	    xrt_core::pci::get_dev(index)->sysfs_put("", "dev_hotplug_done", err, 1);
+        return;
+    }
+    
+    handle = xclOpen(index, nullptr, XCL_QUIET);
+    // there is no way to return this failure, if any, to user
+    // so just save log in case there is failure here
+    // still need to notify user to avoid hang
+    if (!handle) {
+        std::cerr << "host mem config not recovered" << std::endl;
+	xrt_core::pci::get_dev(index)->sysfs_put("", "dev_hotplug_done", err, 1);
+        return;
+    }
+    std::cout << "host mem reconfig (size: " << hostmem_size << ")..." << std::endl;
+    ret = xclCmaEnable(handle, true, hostmem_size);
+    std::cout << "host mem reconfig: " << ret << std::endl;
+    xclClose(handle);
+    
+    //tell the user xclbin load ioctl can be re-issued now
+    xrt_core::pci::get_dev(index)->sysfs_put("", "dev_hotplug_done", err, 1);
+}
+
+/*
+This thread sets the clock frequencies dynamically using aws_clkgen_set_dynamic() API.
+The intention to set these frequencies after image load is to perform clock module reset
+in the back ground which is a work around required to overcome the double image load Hang issue
+observed on the AWS F2 Instance.
+
+NOTE: This Thread has to run Asynchronously in order to avoid the double load Instance Hang issue that
+is seen on F2 instance. The delay 10 secs can be removed once the double load issue on AWS instance is
+addressed by AWS team.
+*/
+static void aws_clkgen_set_dynamic_thread(int slot_id, uint32_t clk_b_freq, uint32_t clk_c_freq, uint32_t clk_hbm_freq)
+{
+    int ret = 0;
+
+    std::this_thread::sleep_for(std::chrono::seconds(10)); // Wait for 10 seconds
+    ret = aws_clkgen_set_dynamic(slot_id, 125 /*default max */, clk_b_freq,
+                            clk_c_freq, clk_hbm_freq, 0);
+    if (ret != 0)
+        std::cerr << "aws_clkgen_set_dynamic_freq failed" << std::endl;
+}
+#endif
 /*
  * On AWS F1, fpga user PF without xclbin being loaded (cleared) has different
  * device id (0x1042) than that of the user PF with xclbin being loaded (0xf010)
- * Changint the device id needs pci node remove and rescan.
+ * Changing the device id needs pci node remove and rescan.
  * fpga_pci_rescan_slot_app_pfs( mBoardNumber ) is the function used to do that.
- * removal of the pf requires unload the xocl driver, within mpd it is impossible.
- * So we assume the user already made the change by loading a default afi from cmdline
- * with fpga-load-local-image.
- * From mpd & xocl perspective, whichever device id doesn't matter.
+ * Doing so in the context of xclbin download ioctl is impossible.
+ * What we are doing here is, if pcie removal & rescan is required, we will return
+ * EAGAIN to userload, at the same time, we do removal & rescan in a separate thread,
+ * Userload code (shim), when seeing EAGAIN, will retry the xclbin download after
+ * the device id has been changed.
+ * This is all transparent to end user.
  */
 int AwsDev::awsLoadXclBin(const xclBin *buffer)
 {
@@ -473,40 +534,85 @@ int AwsDev::awsLoadXclBin(const xclBin *buffer)
     xclmgmt_ioc_bitstream_axlf obj = { const_cast<axlf *>(buffer) };
     return ioctl(mMgtHandle, cmd, &obj);
 #else
-    int retVal = 0;
     axlf *axlfbuffer = reinterpret_cast<axlf*>(const_cast<xclBin*> (buffer));
-    fpga_mgmt_image_info orig_info;
     char* afi_id = get_afi_from_axlf(axlfbuffer);
 
     if (!afi_id)
         return -EINVAL;
 
-    std::memset(&orig_info, 0, sizeof(struct fpga_mgmt_image_info));
-    fpga_mgmt_describe_local_image(mBoardNumber, &orig_info, 0);
-    
-    if (checkAndSkipReload(afi_id, &orig_info)) {
-        // force data retention option
-        union fpga_mgmt_load_local_image_options opt;
-        fpga_mgmt_init_load_local_image_options(&opt);
+    // get old image info before loading new
+    // new image info can only be achieved after being loaded
+    fpga_mgmt_image_info imageInfoOld;
+    fpga_mgmt_describe_local_image(mBoardNumber, &imageInfoOld, 0);
+
+    int retVal = 0;
+    union fpga_mgmt_load_local_image_options opt = {0};
+    // force data retention option
+    fpga_mgmt_init_load_local_image_options(&opt);
+    /* Set the data retention mode only for AWS F1, this is not supported for AWS F2 */
+    if (imageInfoOld.spec.map[FPGA_APP_PF].device_id == 0x1042)
         opt.flags = FPGA_CMD_DRAM_DATA_RETENTION;
-        opt.afi_id = afi_id;
-        opt.slot_id = mBoardNumber;
-        retVal = fpga_mgmt_load_local_image_with_options(&opt);
-        if (retVal == FPGA_ERR_DRAM_DATA_RETENTION_NOT_POSSIBLE ||
-            retVal == FPGA_ERR_DRAM_DATA_RETENTION_FAILED ||
-            retVal == FPGA_ERR_DRAM_DATA_RETENTION_SETUP_FAILED) {
-            std::cout << "INFO: Could not load AFI for data retention, code: " << retVal 
-                      << " - Loading in classic mode." << std::endl;
-            retVal = fpga_mgmt_load_local_image(mBoardNumber, afi_id);
+    opt.afi_id = afi_id;
+    opt.slot_id = mBoardNumber;
+    retVal = fpga_mgmt_load_local_image_with_options(&opt);
+
+    if(imageInfoOld.spec.map[FPGA_APP_PF].device_id == 0xf010)
+    {
+	const clock_freq_topology* clockSection = get_clock_freq_from_axlf(axlfbuffer);
+	uint32_t clock_mains[4];
+
+        if(clockSection) {
+		for(int32_t i = 0; i < clockSection->m_count; i++) {
+                        const struct clock_freq* clk = &(clockSection->m_clock_freq[i]);
+                        clock_mains[i] = clk->m_freq_Mhz;
+                }
         }
-        // check retVal from image load
-        if (retVal) {
-            std::cout << "Failed to load AFI, error: " << retVal << std::endl;
-            return -retVal;
-        }
-        retVal = sleepUntilLoaded( std::string(afi_id) );
+        syslog(LOG_ERR, "WA to skip id=%x", imageInfoOld.spec.map[FPGA_APP_PF].device_id);
+#ifndef INTERNAL_TESTING_FOR_AWS
+        auto future = std::async(std::launch::async, aws_clkgen_set_dynamic_thread, mBoardNumber, clock_mains[2]/*clk_extra_b0*/, clock_mains[3]/*clk_extra_c0*/, clock_mains[1]/*hbm*/);
+#endif
+        return 0;
+     }
+
+    if (retVal == FPGA_ERR_DRAM_DATA_RETENTION_NOT_POSSIBLE ||
+        retVal == FPGA_ERR_DRAM_DATA_RETENTION_FAILED ||
+        retVal == FPGA_ERR_DRAM_DATA_RETENTION_SETUP_FAILED) {
+        std::cout << "INFO: Could not load AFI for data retention, code: " << retVal 
+                  << " - Loading in classic mode." << std::endl;
+        retVal = fpga_mgmt_load_local_image(mBoardNumber, afi_id);
     }
-    return retVal;
+    // check retVal from image load
+    if (retVal) {
+        std::cerr << "Failed to load AFI, error: " << retVal << std::endl;
+        return -retVal;
+    }
+
+    fpga_mgmt_image_info imageInfoNew;
+    retVal = sleepUntilLoaded(std::string(afi_id), &imageInfoNew);
+    if (retVal) {
+        std::cerr << "Failed to load AFI, error: " << retVal << std::endl;
+        return -retVal;
+    }
+
+    // if there is device id change, or shell version change (2rp case), we do a rescan
+    // at the same time, we ask the user (shim) to reload the 2nd time later
+    std::cout << "device id before load: 0x" <<  std::hex << imageInfoOld.spec.map[FPGA_APP_PF].device_id << std::endl;
+    std::cout << "device id after load: 0x" <<  imageInfoNew.ids.afi_device_ids.device_id << std::endl;
+    std::cout << "shell version before load: 0x" <<  imageInfoOld.sh_version << std::endl;
+    std::cout << "shell version after load: 0x" <<  imageInfoNew.sh_version << std::dec << std::endl;
+    if (imageInfoOld.spec.map[FPGA_APP_PF].device_id != imageInfoNew.ids.afi_device_ids.device_id ||
+        imageInfoOld.sh_version != imageInfoNew.sh_version) {
+        std::cout << "pci removal & rescan..." << std::endl;
+        std::string err;
+	xrt_core::pci::get_dev(index)->sysfs_put("", "dev_hotplug_done", err, 0);
+	    
+        if (rescan_thread[mBoardNumber].joinable())
+                rescan_thread[mBoardNumber].join();
+        rescan_thread[mBoardNumber]  = std::thread(awsPciRescan, index);
+        return -EAGAIN;
+    }
+    
+    return 0;
 #endif
 }
 
@@ -530,13 +636,14 @@ int AwsDev::awsGetIcap(xcl_pr_region *data)
 #else
     fpga_mgmt_image_info imageInfo;
     fpga_mgmt_describe_local_image( mBoardNumber, &imageInfo, 0 );
-    FIELD(data, freq, 0) = imageInfo.metrics.clocks[0].frequency[0] / 1000000;
-    FIELD(data, freq, 1) = imageInfo.metrics.clocks[1].frequency[0] / 1000000;
-    FIELD(data, freq, 2) = imageInfo.metrics.clocks[2].frequency[0] / 1000000;
-    FIELD(data, freq_cntr, 0) = imageInfo.metrics.clocks[0].frequency[0] / 1000;
-    FIELD(data, freq_cntr, 1) = imageInfo.metrics.clocks[1].frequency[0] / 1000;
-    FIELD(data, freq_cntr, 2) = imageInfo.metrics.clocks[2].frequency[0] / 1000;
+    FIELD(data, freq, 0) = imageInfo.metrics.f2_metrics.clocks[0].frequency[0] / 1000000;
+    FIELD(data, freq, 1) = imageInfo.metrics.f2_metrics.clocks[1].frequency[0] / 1000000;
+    FIELD(data, freq, 2) = imageInfo.metrics.f2_metrics.clocks[2].frequency[0] / 1000000;
+    FIELD(data, freq_cntr, 0) = imageInfo.metrics.f2_metrics.clocks[0].frequency[0] / 1000;
+    FIELD(data, freq_cntr, 1) = imageInfo.metrics.f2_metrics.clocks[1].frequency[0] / 1000;
+    FIELD(data, freq_cntr, 2) = imageInfo.metrics.f2_metrics.clocks[2].frequency[0] / 1000;
     data->data_retention = 1;
+    data->idcode = imageInfo.sh_version;
 #endif
     //do we need to save uuid of xclbin loaded so that we can return xclbin uuid here?
     //seems not. we check afi before load new xclbin.
@@ -591,19 +698,7 @@ bool AwsDev::isGood() {
 
 int AwsDev::awsUserProbe(xcl_mailbox_conn_resp *resp)
 {
-#ifndef INTERNAL_TESTING_FOR_AWS
-    fpga_slot_spec spec_array[16];
-    std::memset(spec_array, 0, sizeof(fpga_slot_spec) * 16);
-    if (fpga_pci_get_all_slot_specs(spec_array, 16)) {
-        std::cout << "ERROR: failed at fpga_pci_get_all_slot_specs" << std::endl;
-        return -1;
-    }
-
-    if (spec_array[mBoardNumber].map[FPGA_APP_PF].device_id != AWS_UserPF_DEVICE_ID)
-        resp->conn_flags |= XCL_MB_PEER_READY;
-#else
     resp->conn_flags |= XCL_MB_PEER_READY;
-#endif
     return 0;
 }
 
@@ -672,7 +767,7 @@ AwsDev::AwsDev(size_t index, const char *logfileName)
         mLogStream << __func__ << ", " << std::this_thread::get_id() << std::endl;
     }
 
-    std::string sysfs_name = pcidev::get_dev(index, true)->sysfs_name;
+    std::string sysfs_name = xrt_core::pci::get_dev(index, true)->m_sysfs_name;
     std::cout << "AwsDev: " << sysfs_name << "(index: " << index << ")" << std::endl;
 #ifdef INTERNAL_TESTING_FOR_AWS
     mBoardNumber = index;
@@ -688,16 +783,17 @@ AwsDev::AwsDev(size_t index, const char *logfileName)
 #else
     fpga_mgmt_init(); // aws-fpga version newer than 09/2019 need this
     mBoardNumber = index_map[sysfs_name];
+    this->index = index;
     //bar0 is mapped already. seems other 2 bars are not required.
 #endif
 }
 
 //private functions
 #ifndef INTERNAL_TESTING_FOR_AWS
-int AwsDev::sleepUntilLoaded( const std::string &afi )
+int AwsDev::sleepUntilLoaded( const std::string &afi, fpga_mgmt_image_info *image_info )
 {
     for( int i = 0; i < 20; i++ ) {
-        std::this_thread::sleep_for( std::chrono::milliseconds( 100 ) );
+        std::this_thread::sleep_for( std::chrono::milliseconds( 500 ) );
         fpga_mgmt_image_info info;
         std::memset( &info, 0, sizeof(struct fpga_mgmt_image_info) );
         int result = fpga_mgmt_describe_local_image( mBoardNumber, &info, 0 );
@@ -706,55 +802,13 @@ int AwsDev::sleepUntilLoaded( const std::string &afi )
             return 1;
         }
         if( (info.status == FPGA_STATUS_LOADED) && !std::strcmp(info.ids.afi_id, const_cast<char*>(afi.c_str())) ) {
+            *image_info = info;
             break;
         }
     }
     return 0;
 }
 
-int AwsDev::checkAndSkipReload( char *afi_id, fpga_mgmt_image_info *orig_info )
-{
-    if( (orig_info->status == FPGA_STATUS_LOADED) && !std::strcmp(orig_info->ids.afi_id, afi_id) ) {
-        std::cout << "This AFI already loaded. Skip reload!" << std::endl;
-        int result = 0;
-        //existing afi matched.
-        uint16_t status = 0;
-        result = fpga_mgmt_get_vDIP_status(mBoardNumber, &status);
-        if(result) {
-            std::cout << "Error: can not get virtual DIP Switch state" << std::endl;
-            return result;
-        }
-        //Set bit 0 to 1
-        status |=  (1 << 0);
-        result = fpga_mgmt_set_vDIP(mBoardNumber, status);
-        if(result) {
-            std::cout << "Error trying to set virtual DIP Switch" << std::endl;
-            return result;
-        }
-        std::this_thread::sleep_for(std::chrono::microseconds(250));
-        //pulse the changes in.
-        result = fpga_mgmt_get_vDIP_status(mBoardNumber, &status);
-        if(result) {
-            std::cout << "Error: can not get virtual DIP Switch state" << std::endl;
-            return result;
-        }
-        //Set bit 0 to 0
-        status &=  ~(1 << 0);
-        result = fpga_mgmt_set_vDIP(mBoardNumber, status);
-        if(result) {
-            std::cout << "Error trying to set virtual DIP Switch" << std::endl;
-            return result;
-        }
-        std::this_thread::sleep_for(std::chrono::microseconds(250));
-        
-        std::cout << "Successfully skipped reloading of local image." << std::endl;
-        return result;
-    } else {
-        std::cout << "AFI not yet loaded, proceed to download." << std::endl;
-        return 1;
-    }
-}
-
 char *AwsDev::get_afi_from_axlf(const axlf *buffer)
 {
     const axlf_section_header *bit_header = xclbin::get_axlf_section(buffer, BITSTREAM);
@@ -766,4 +820,14 @@ char *AwsDev::get_afi_from_axlf(const axlf *buffer)
         return nullptr;
     return afid;
 }
+
+const clock_freq_topology* AwsDev::get_clock_freq_from_axlf(const axlf *buffer)
+{
+    const axlf_section_header *clk_header = xclbin::get_axlf_section(buffer, CLOCK_FREQ_TOPOLOGY);
+    char *clk_topo = const_cast<char *>(reinterpret_cast<const char *>(buffer));
+    clk_topo += clk_header->m_sectionOffset;
+
+    return (reinterpret_cast<const clock_freq_topology*>(clk_topo));
+}
+
 #endif
