@@ -18,7 +18,7 @@
  * Embedded runtime scheduler
  */
 
-#include "core/include/ert.h"
+#include "core/include/xrt/detail/ert.h"
 // includes from bsp
 #ifndef ERT_HW_EMU
 #include <xil_printf.h>
@@ -33,6 +33,7 @@
 #include <stdlib.h>
 #include <limits>
 #include <bitset>
+#include <cstring>
 
 // version is a git hash passed in from build script
 // default for builds that bypass build script
@@ -199,6 +200,8 @@ static value_type mb_host_interrupt_enabled = 0;
 static value_type cu_dma_52                 = 0;
 static value_type cdma_enabled              = 0;
 static value_type dataflow_enabled          = 0;
+static value_type kds_30                    = 0;
+static value_type echo                      = 0;
 
 // Struct slot_info is per command slot in command queue
 struct slot_info
@@ -227,6 +230,8 @@ struct slot_info
   size_type regmap_size = 0;
 };
 
+static value_type slot_cache[max_slots];
+
 // Fixed sized map from slot_idx -> slot info
 static slot_info command_slots[max_slots];
 
@@ -240,6 +245,9 @@ static size_type cu_usage[max_cus];
 // Only 'num_cus' lower bits are used
 static bitset_type cu_status;
 
+// Bitmask indicates the slot is submitted or not: (0) not submitted, (1) submitted
+static bitset_type slot_submitted;
+
 // Bitmask for interrupt enabled CUs.  (0) no interrupt (1) enabled
 static bitset_type cu_interrupt_mask;
 #ifndef ERT_HW_EMU
@@ -424,9 +432,12 @@ setup()
 {
   CTRL_DEBUG("-> setup\n");
 
-  // In dataflow number of slots is number of CUs plus ctrl slot (0),
+  // In dataflow mode, the number of slots of kds 2.0 is number of CUs plus ctrl slot (0) 
+  // the number of slot of kds 3.0 as many slots as possible per slot_size
   // otherwise its as many slots as possible per slot_size
-  num_slots = dataflow_enabled ? num_cus+1 : ERT_CQ_SIZE / slot_size;
+  num_slots = dataflow_enabled ? 
+                  kds_30 ? ERT_CQ_SIZE / slot_size : num_cus+1 
+                  : ERT_CQ_SIZE / slot_size;
   num_slot_masks = ((num_slots-1)>>5) + 1;
   num_cu_masks = ((num_cus-1)>>5) + 1;
 
@@ -456,6 +467,7 @@ setup()
 
     // Clear command queue headers memory
     write_reg(slot.slot_addr,0x0);
+    slot_cache[i] = 0;
   }
 
   // Clear CSR  (COR so read)
@@ -463,6 +475,7 @@ setup()
     ERT_UNUSED volatile auto val = read_reg(STATUS_REGISTER_ADDR[i]);
 
   cu_status.reset();
+  slot_submitted.reset();
 
   // Initialize cu_slot_usage
   for (size_type i=0; i<num_cus; ++i) {
@@ -625,7 +638,6 @@ configure_cu(addr_type cu_addr, addr_type regmap_addr, size_type regmap_size)
   // 0x4, 0x8, 0xc used for interrupt, which is initialized in setup
   for (size_type idx = 4; idx < regmap_size; ++idx)
     write_reg(cu_addr + (idx << 2), read_reg(regmap_addr + (idx << 2)));
-
   // start kernel at base + 0x0
   write_reg(cu_addr, 0x1);
 }
@@ -637,8 +649,7 @@ inline void
 configure_cu_ooo(addr_type cu_addr, addr_type regmap_addr, size_type regmap_size)
 {
   // write register map addr, value pairs starting 
-  // past reserved 4 ctrl + 2 ctx 
-  for (size_type idx = 6; idx < regmap_size; idx += 2) {
+  for (size_type idx = 0; idx < regmap_size; idx += 2) {
     addr_type offset = read_reg(regmap_addr + (idx << 2));
     value_type value = read_reg(regmap_addr + ((idx + 1) << 2));
     write_reg(cu_addr + offset, value);
@@ -823,6 +834,8 @@ configure_mb(size_type slot_idx)
   cq_status_enabled = (features & 0x10)!=0;
   cdma_enabled = (features & 0x20)!=0;
   dataflow_enabled = (features & 0x40)!=0;
+  kds_30 = (features & 0x100)!=0;
+  echo = (features & 0x400)!=0;
 #ifndef ERT_HW_EMU
   cu_dma_52 = (features & 0x80000000)!=0;
 #else
@@ -1100,7 +1113,78 @@ running_to_free(size_type slot_idx)
 #endif
   return true;
 }
+static inline void
+command_queue_fetch(size_type slot_idx)
+{
+  auto& slot = command_slots[slot_idx];
+  value_type slot_addr = slot.slot_addr;
+  auto val = read_reg(slot_addr);
+ 
+  /* TODO: Workaround for the BRAM read/write collision HW issue,
+   * which will lead to ERT got incorrect command header.
+   *
+   * If command slot header is not zero, read command header again.
+   * The second read will return the correct value.
+   */
+  if (val != 0)
+      val = read_reg(slot_addr);
+  /* TODO: Work around ends */
 
+  if (val & AP_START) {
+    write_reg(slot_addr,0x0);// clear command queue
+    if (echo) {
+      notify_host(slot_idx);
+      return;
+    }
+
+    slot_cache[slot_idx] = val;
+    addr_type addr = cu_section_addr(slot_addr);
+    slot.cu_idx = read_reg(addr);
+    slot.opcode = opcode(val);
+    slot.header_value = val;
+    slot.regmap_addr = regmap_section_addr(val,slot_addr);
+    slot.regmap_size = regmap_size(val);
+  }
+}
+
+static inline void
+cu_state_check(size_type slot_idx)
+{
+  auto& slot = command_slots[slot_idx];
+  auto cuvalue = read_reg(cu_idx_to_addr(slot.cu_idx));
+
+  // check this CU if done
+  if (cu_status[slot.cu_idx]) {
+    if (cuvalue & (AP_DONE)) {
+      auto cu_slot = cu_slot_usage[slot.cu_idx];
+
+      write_reg(cu_idx_to_addr(slot.cu_idx), AP_CONTINUE);
+      notify_host(cu_slot);
+      cu_status[slot.cu_idx] = !cu_status[slot.cu_idx];// now cu is available for next cmd
+      slot_cache[cu_slot] = 0; // This slot have been submitted and completed, free it
+    }
+  }
+}
+
+static inline void
+cu_execution(size_type slot_idx)
+{
+  auto& slot = command_slots[slot_idx];
+
+  if (!cu_status[slot.cu_idx]) {
+    if (slot_cache[slot_idx] & AP_START) {
+
+      if (slot.opcode==ERT_EXEC_WRITE) // Out of order configuration
+        configure_cu_ooo(cu_idx_to_addr(slot.cu_idx),slot.regmap_addr,slot.regmap_size);
+      else
+        configure_cu(cu_idx_to_addr(slot.cu_idx),slot.regmap_addr,slot.regmap_size);
+
+      cu_status[slot.cu_idx] = !cu_status[slot.cu_idx];
+      set_cu_info(slot.cu_idx,slot_idx); // record which slot cu associated with
+
+    }
+  }
+}
 /**
  * Main routine executed by embedded scheduler loop
  *
@@ -1134,11 +1218,10 @@ scheduler_loop()
 #ifdef ERT_HW_EMU
       reg_access_wait();
 #endif
-
       // In dataflow mode ERT is polling CUs for completion after
       // host has started CU or acknowleged completion.  Ctrl cmds
       // are processed in normal flow.
-      if (dataflow_enabled && slot_idx>0) {
+      if (dataflow_enabled && slot_idx>0 && !kds_30) {
         size_type cuidx = slot_idx-1;  // compensate for reserved slot (0)
 
         // Check if host has started or continued this CU
@@ -1175,8 +1258,21 @@ scheduler_loop()
         continue;
       }
 
-      // CQ_STATUS_ENABLED CHECK WON'T WORK IF HOST TRANSITIONS
-      // FROM ENABLED -> DISABLED IN CONFIGURE COMMAND
+      if (dataflow_enabled && slot_idx>0 && kds_30) {
+        if (!slot_cache[slot_idx])
+            command_queue_fetch(slot_idx);
+
+        // we have nothing else to do
+        if (!slot_cache[slot_idx])
+          continue;
+
+        cu_state_check(slot_idx);
+
+        cu_execution(slot_idx);
+        
+        continue;
+      }
+
       if (!cq_status_enabled && ((slot.header_value & 0xF) == 0x4)) { // free
         if (!free_to_new(slot_idx))
           continue;
