@@ -1,5 +1,6 @@
 /**
- * Copyright (C) 2019 Xilinx, Inc
+ * Copyright (C) 2019-2022 Xilinx, Inc
+ * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License"). You may
  * not use this file except in compliance with the License. A copy of the
@@ -17,44 +18,84 @@
 #include "SectionSoftKernel.h"
 
 #include "XclBinUtilities.h"
-namespace XUtil = XclBinUtilities;
-
 #include <boost/algorithm/string.hpp>
+#include <boost/format.hpp>
+#include <boost/functional/factory.hpp>
 #include <boost/property_tree/json_parser.hpp>
 
+namespace XUtil = XclBinUtilities;
+
 // Disable windows compiler warnings
 #ifdef _WIN32
 #pragma warning( disable : 4100 4267 4244)
 #endif
 
 // Static Variables / Classes
-SectionSoftKernel::_init SectionSoftKernel::_initializer;
+SectionSoftKernel::init SectionSoftKernel::initializer;
 
-// -------------------------------------------------------------------------
+SectionSoftKernel::init::init()
+{
+  auto sectionInfo = std::make_unique<SectionInfo>(SOFT_KERNEL, "SOFT_KERNEL", boost::factory<SectionSoftKernel*>());
+  sectionInfo->supportsSubSections = true;
+  sectionInfo->subSections.push_back(getSubSectionName(SubSection::obj));
+  sectionInfo->subSections.push_back(getSubSectionName(SubSection::metadata));
 
-SectionSoftKernel::SectionSoftKernel() {
-  // Empty
-}
+  sectionInfo->supportsIndexing = true;
 
-// -------------------------------------------------------------------------
-
-SectionSoftKernel::~SectionSoftKernel() {
-  // Empty
-}
-
-// -------------------------------------------------------------------------
-
-bool
-SectionSoftKernel::doesSupportAddFormatType(FormatType _eFormatType) const {
-  // The Soft Kernel top-level section does support any add syntax.
+  // Add format support empty (no support)
+  // The top-level section doesn't support any add syntax.
   // Must use sub-sections
-  return false;
+
+  sectionInfo->supportedAddFormats.push_back(FormatType::raw);
+
+  addSectionType(std::move(sectionInfo));
+}
+
+// -------------------------------------------------------------------------
+
+using SubSectionTableCollection = std::vector<std::pair<std::string, SectionSoftKernel::SubSection>>;
+static const SubSectionTableCollection&
+getSubSectionTable()
+{
+  static const SubSectionTableCollection subSectionTable = {
+    { "UNKNOWN", SectionSoftKernel::SubSection::unknown },
+    { "OBJ", SectionSoftKernel::SubSection::obj },
+    { "METADATA", SectionSoftKernel::SubSection::metadata }
+  };
+  return subSectionTable;
+}
+
+SectionSoftKernel::SubSection
+SectionSoftKernel::getSubSectionEnum(const std::string& sSubSectionName)
+{
+  auto subSectionTable = getSubSectionTable();
+  auto iter = std::find_if(subSectionTable.begin(), subSectionTable.end(), [&](const auto& entry) {return boost::iequals(entry.first, sSubSectionName);});
+
+  if (iter == subSectionTable.end())
+    return SubSection::unknown;
+
+  return iter->second;
+}
+
+// -------------------------------------------------------------------------
+
+const std::string&
+SectionSoftKernel::getSubSectionName(SectionSoftKernel::SubSection eSubSection)
+{
+  const auto& subSectionTable = getSubSectionTable();
+  auto iter = std::find_if(subSectionTable.begin(), subSectionTable.end(), [&](const auto& entry) {return entry.second == eSubSection;});
+
+  if (iter == subSectionTable.end())
+    return getSubSectionName(SubSection::unknown);
+
+  return iter->first;
 }
 
 // -------------------------------------------------------------------------
 
 bool
-SectionSoftKernel::subSectionExists(const std::string& _sSubSectionName) const {
+SectionSoftKernel::subSectionExists(const std::string& _sSubSectionName) const
+{
 
   // No buffer no subsections
   if (m_pBuffer == nullptr) {
@@ -62,14 +103,14 @@ SectionSoftKernel::subSectionExists(const std::string& _sSubSectionName) const {
   }
 
   // There is a sub-system
-  
+
   // Determine if the metadata section has been initialized by the user.
   // If not then it doesn't really exist
-  
+
   // Extract the sub-section entry type
   SubSection eSS = getSubSectionEnum(_sSubSectionName);
 
-  if (eSS == SS_METADATA) {
+  if (eSS == SubSection::metadata) {
     // Extract the binary data as a JSON string
     std::ostringstream buffer;
     writeMetadata(buffer);
@@ -100,73 +141,47 @@ SectionSoftKernel::subSectionExists(const std::string& _sSubSectionName) const {
 
 // -------------------------------------------------------------------------
 
-bool
-SectionSoftKernel::supportsSubSection(const std::string& _sSubSectionName) const {
-  if (getSubSectionEnum(_sSubSectionName) == SS_UNKNOWN) {
-    return false;
-  }
-
-  return true;
-}
-
-// -------------------------------------------------------------------------
-
-enum SectionSoftKernel::SubSection
-SectionSoftKernel::getSubSectionEnum(const std::string _sSubSectionName) const {
-
-  // Case-insensitive
-  std::string sSubSection = _sSubSectionName;
-  boost::to_upper(sSubSection);
-
-  // Convert string to the enumeration value
-  if (sSubSection == "OBJ") {return SS_OBJ;}
-  if (sSubSection == "METADATA") {return SS_METADATA;}
-
-  return SS_UNKNOWN;
-}
-
-
-// -------------------------------------------------------------------------
 void
 SectionSoftKernel::copyBufferUpdateMetadata(const char* _pOrigDataSection,
                                             unsigned int _origSectionSize,
-                                            std::fstream& _istream,
-                                            std::ostringstream& _buffer) const {
+                                            std::istream& _istream,
+                                            std::ostringstream& _buffer) const
+{
   XUtil::TRACE("SectionSoftKernel::CopyBufferUpdateMetadata");
 
   // Do we have enough room to overlay the header structure
   if (_origSectionSize < sizeof(soft_kernel)) {
-    std::string errMsg = XUtil::format("ERROR: Segment size (%d) is smaller than the size of the soft_kernel structure (%d)", _origSectionSize, sizeof(soft_kernel));
-    throw std::runtime_error(errMsg);
+    auto errMsg = boost::format("ERROR: Segment size (%d) is smaller than the size of the soft_kernel structure (%d)") % _origSectionSize % sizeof(soft_kernel);
+    throw std::runtime_error(errMsg.str());
   }
 
   // Prepare our destination header buffer
   soft_kernel softKernelHdr = { 0 };      // Header buffer
   std::ostringstream stringBlock;         // String block (stored immediately after the header)
 
-  const soft_kernel* pHdr = reinterpret_cast<const soft_kernel*>(_pOrigDataSection);
+  auto pHdr = reinterpret_cast<const soft_kernel*>(_pOrigDataSection);
 
   XUtil::TRACE_BUF("soft_kernel-original", reinterpret_cast<const char*>(pHdr), sizeof(soft_kernel));
-  XUtil::TRACE(XUtil::format("Original: \n"
+  XUtil::TRACE(boost::format("Original: \n"
                              "  mpo_name (0x%lx): '%s'\n"
                              "  m_image_offset: 0x%lx, m_image_size: 0x%lx\n"
                              "  mpo_version (0x%lx): '%s'\n"
                              "  mpo_md5_value (0x%lx): '%s'\n"
                              "  mpo_symbol_name (0x%lx): '%s'\n"
-                             "  m_num_instances: %d",
-                             pHdr->mpo_name, reinterpret_cast<const char*>(pHdr) + pHdr->mpo_name,
-                             pHdr->m_image_offset, pHdr->m_image_size,
-                             pHdr->mpo_version, reinterpret_cast<const char*>(pHdr) + pHdr->mpo_version,
-                             pHdr->mpo_md5_value, reinterpret_cast<const char*>(pHdr) + pHdr->mpo_md5_value,
-                             pHdr->mpo_symbol_name, reinterpret_cast<const char*>(pHdr) + pHdr->mpo_symbol_name,
-                             pHdr->m_num_instances));
+                             "  m_num_instances: %d")
+               % pHdr->mpo_name % (reinterpret_cast<const char*>(pHdr) + pHdr->mpo_name)
+               % pHdr->m_image_offset % pHdr->m_image_size
+               % pHdr->mpo_version % (reinterpret_cast<const char*>(pHdr) + pHdr->mpo_version)
+               % pHdr->mpo_md5_value % (reinterpret_cast<const char*>(pHdr) + pHdr->mpo_md5_value)
+               % pHdr->mpo_symbol_name % (reinterpret_cast<const char*>(pHdr) + pHdr->mpo_symbol_name)
+               % pHdr->m_num_instances);
 
   // Get the JSON metadata
   _istream.seekg(0, _istream.end);             // Go to the beginning
   std::streampos fileSize = _istream.tellg();  // Go to the end
 
   // Copy the buffer into memory
-  std::unique_ptr<unsigned char> memBuffer(new unsigned char[fileSize]);
+  std::unique_ptr<unsigned char[]> memBuffer(new unsigned char[fileSize]);
   _istream.clear();                                // Clear any previous errors
   _istream.seekg(0);                               // Go to the beginning
   _istream.read((char*)memBuffer.get(), fileSize); // Read in the buffer into memory
@@ -189,44 +204,52 @@ SectionSoftKernel::copyBufferUpdateMetadata(const char* _pOrigDataSection,
 
   // mpo_name
   {
-    std::string sDefault = reinterpret_cast<const char*>(pHdr) + sizeof(soft_kernel) + pHdr->mpo_name;
-    std::string sValue = ptSK.get<std::string>("mpo_name", sDefault);
+    auto sDefault = reinterpret_cast<const char*>(pHdr) + sizeof(soft_kernel) + pHdr->mpo_name;
+    auto sValue = ptSK.get<std::string>("mpo_name", sDefault);
 
     if (sValue.compare(getSectionIndexName()) != 0) {
-      std::string errMsg = XUtil::format("ERROR: Metadata data mpo_name '%s' does not match expected section name '%s'", sValue.c_str(), getSectionIndexName().c_str());
-      throw std::runtime_error(errMsg);
+      auto errMsg = boost::format("ERROR: Metadata data mpo_name '%s' does not match expected section name '%s'") % sValue % getSectionIndexName();
+      throw std::runtime_error(errMsg.str());
     }
 
     softKernelHdr.mpo_name = sizeof(soft_kernel) + stringBlock.tellp();
-    stringBlock << sValue << '\0';   
-    XUtil::TRACE(XUtil::format("  mpo_name (0x%lx): '%s'", softKernelHdr.mpo_name, sValue.c_str()).c_str());
+    stringBlock << sValue << '\0';
+    XUtil::TRACE(boost::format("  mpo_name (0x%lx): '%s'") % softKernelHdr.mpo_name % sValue);
   }
 
   // mpo_version
   {
-    std::string sDefault = reinterpret_cast<const char*>(pHdr) + sizeof(soft_kernel) + pHdr->mpo_version;
-    std::string sValue = ptSK.get<std::string>("mpo_version", sDefault);
+    auto sDefault = reinterpret_cast<const char*>(pHdr) + sizeof(soft_kernel) + pHdr->mpo_version;
+    auto sValue = ptSK.get<std::string>("mpo_version", sDefault);
     softKernelHdr.mpo_version = sizeof(soft_kernel) + stringBlock.tellp();
     stringBlock << sValue << '\0';
-    XUtil::TRACE(XUtil::format("  mpo_version (0x%lx): '%s'", softKernelHdr.mpo_version, sValue.c_str()).c_str());
+    XUtil::TRACE(boost::format("  mpo_version (0x%lx): '%s'") % softKernelHdr.mpo_version % sValue);
   }
 
   // mpo_md5_value
   {
-    std::string sDefault = reinterpret_cast<const char*>(pHdr) + sizeof(soft_kernel) + pHdr->mpo_md5_value;
-    std::string sValue = ptSK.get<std::string>("mpo_md5_value", sDefault);
+    auto sDefault = reinterpret_cast<const char*>(pHdr) + sizeof(soft_kernel) + pHdr->mpo_md5_value;
+    auto sValue = ptSK.get<std::string>("mpo_md5_value", sDefault);
     softKernelHdr.mpo_md5_value = sizeof(soft_kernel) + stringBlock.tellp();
     stringBlock << sValue << '\0';
-    XUtil::TRACE(XUtil::format("  mpo_md5_value (0x%lx): '%s'", softKernelHdr.mpo_md5_value, sValue.c_str()).c_str());
+    XUtil::TRACE(boost::format("  mpo_md5_value (0x%lx): '%s'") % softKernelHdr.mpo_md5_value % sValue);
   }
 
   // mpo_symbol_name
   {
-    std::string sDefault = reinterpret_cast<const char*>(pHdr) + sizeof(soft_kernel) + pHdr->mpo_symbol_name;
-    std::string sValue = ptSK.get<std::string>("mpo_symbol_name", sDefault);
+    auto sDefault = reinterpret_cast<const char*>(pHdr) + sizeof(soft_kernel) + pHdr->mpo_symbol_name;
+    auto sValue = ptSK.get<std::string>("mpo_symbol_name", sDefault);
     softKernelHdr.mpo_symbol_name = sizeof(soft_kernel) + stringBlock.tellp();
     stringBlock << sValue << '\0';
-    XUtil::TRACE(XUtil::format("  mpo_symbol_name (0x%lx): '%s'", softKernelHdr.mpo_symbol_name, sValue.c_str()).c_str());
+    XUtil::TRACE(boost::format("  mpo_symbol_name (0x%lx): '%s'") % softKernelHdr.mpo_symbol_name % sValue);
+
+    // DRC Check checking to see if the maximum symbol name length has been violated
+    constexpr unsigned int MAX_SYMBOL_NAME_LENGTH = 19;
+    if (sValue.length() > MAX_SYMBOL_NAME_LENGTH) {
+      auto errMsg = boost::format("ERROR: The given symbol name '%s' (length %d) exceeds the maximum support length of %d characters.")
+          % sValue % sValue.length() % MAX_SYMBOL_NAME_LENGTH;
+      throw std::runtime_error(errMsg.str());
+    }
   }
 
   // m_num_instances
@@ -234,15 +257,22 @@ SectionSoftKernel::copyBufferUpdateMetadata(const char* _pOrigDataSection,
     uint32_t defaultValue = pHdr->m_num_instances;
     uint32_t value = ptSK.get<uint32_t>("m_num_instances", defaultValue);
     softKernelHdr.m_num_instances = value;
-    XUtil::TRACE(XUtil::format("  m_num_instances: %d", softKernelHdr.m_num_instances).c_str());
+    XUtil::TRACE(boost::format("  m_num_instances: %d") % softKernelHdr.m_num_instances);
+
+    // DRC Check checking the maximum number of instances
+    constexpr unsigned int MAX_NUM_INSTANCES = 128;
+    if (value > MAX_NUM_INSTANCES) {
+      auto errMsg = boost::format("ERROR: The number of instances (%d) exceeds the maximum supported value (%d).") % value % MAX_NUM_INSTANCES;
+      throw std::runtime_error(errMsg.str());
+    }
   }
 
   // Last item to be initialized
   {
     softKernelHdr.m_image_offset = sizeof(soft_kernel) + stringBlock.tellp();
     softKernelHdr.m_image_size = pHdr->m_image_size;
-    XUtil::TRACE(XUtil::format("  m_image_offset: 0x%lx", softKernelHdr.m_image_offset).c_str());
-    XUtil::TRACE(XUtil::format("    m_image_size: 0x%lx", softKernelHdr.m_image_size).c_str());
+    XUtil::TRACE(boost::format("  m_image_offset: 0x%lx") % softKernelHdr.m_image_offset);
+    XUtil::TRACE(boost::format("    m_image_size: 0x%lx") % softKernelHdr.m_image_size);
   }
 
   // Copy the output to the output buffer.
@@ -260,20 +290,21 @@ SectionSoftKernel::copyBufferUpdateMetadata(const char* _pOrigDataSection,
 // -------------------------------------------------------------------------
 
 void
-SectionSoftKernel::createDefaultImage(std::fstream& _istream, std::ostringstream& _buffer) const {
+SectionSoftKernel::createDefaultImage(std::istream& _istream, std::ostringstream& _buffer) const
+{
   XUtil::TRACE("SOFT_KERNEL-OBJ");
 
-  soft_kernel softKernelHdr = soft_kernel{0};
+  soft_kernel softKernelHdr = soft_kernel{};
   std::ostringstream stringBlock;       // String block (stored immediately after the header)
 
   // Initialize default values
   {
     // Have all of the mpo (member, point, offset) values point to the zero length terminate string
     softKernelHdr.mpo_name = sizeof(soft_kernel) + stringBlock.tellp();
-    stringBlock << getSectionIndexName() << '\0';   
+    stringBlock << getSectionIndexName() << '\0';
 
-    uint32_t mpo_emptyChar = sizeof(soft_kernel) + stringBlock.tellp();   
-    stringBlock << '\0';   
+    uint32_t mpo_emptyChar = sizeof(soft_kernel) + stringBlock.tellp();
+    stringBlock << '\0';
 
     softKernelHdr.mpo_version = mpo_emptyChar;
     softKernelHdr.mpo_md5_value = mpo_emptyChar;
@@ -298,10 +329,10 @@ SectionSoftKernel::createDefaultImage(std::fstream& _istream, std::ostringstream
 
   // Write Data
   {
-    std::unique_ptr<unsigned char> memBuffer(new unsigned char[softKernelHdr.m_image_size]);
+    std::unique_ptr<unsigned char[]> memBuffer(new unsigned char[softKernelHdr.m_image_size]);
     _istream.seekg(0);
     _istream.clear();
-    _istream.read(reinterpret_cast<char *>(memBuffer.get()), softKernelHdr.m_image_size);
+    _istream.read(reinterpret_cast<char*>(memBuffer.get()), softKernelHdr.m_image_size);
 
     _buffer.write(reinterpret_cast<const char*>(memBuffer.get()), softKernelHdr.m_image_size);
   }
@@ -312,22 +343,23 @@ SectionSoftKernel::createDefaultImage(std::fstream& _istream, std::ostringstream
 void
 SectionSoftKernel::readSubPayload(const char* _pOrigDataSection,
                                   unsigned int _origSectionSize,
-                                  std::fstream& _istream,
+                                  std::istream& _istream,
                                   const std::string& _sSubSectionName,
-                                  enum Section::FormatType _eFormatType,
-                                  std::ostringstream& _buffer) const {
+                                  Section::FormatType _eFormatType,
+                                  std::ostringstream& _buffer) const
+{
   // Determine the sub-section of interest
   SubSection eSubSection = getSubSectionEnum(_sSubSectionName);
 
   switch (eSubSection) {
-    case SS_OBJ:
+    case SubSection::obj:
       // Some basic DRC checks
       if (_pOrigDataSection != nullptr) {
         std::string errMsg = "ERROR: Soft kernel object image already exists.";
         throw std::runtime_error(errMsg);
       }
 
-      if (_eFormatType != Section::FT_RAW) {
+      if (_eFormatType != Section::FormatType::raw) {
         std::string errMsg = "ERROR: Soft kernel's object only supports the RAW format.";
         throw std::runtime_error(errMsg);
       }
@@ -335,14 +367,14 @@ SectionSoftKernel::readSubPayload(const char* _pOrigDataSection,
       createDefaultImage(_istream, _buffer);
       break;
 
-    case SS_METADATA: {
+    case SubSection::metadata: {
         // Some basic DRC checks
         if (_pOrigDataSection == nullptr) {
           std::string errMsg = "ERROR: Missing soft kernel object image.  Add the SOFT_KERNEL-OBJ image prior to changing its metadata.";
           throw std::runtime_error(errMsg);
         }
 
-        if (_eFormatType != Section::FT_JSON) {
+        if (_eFormatType != Section::FormatType::json) {
           std::string errMsg = "ERROR: SOFT_KERNEL-METADATA only supports the JSON format.";
           throw std::runtime_error(errMsg);
         }
@@ -351,10 +383,10 @@ SectionSoftKernel::readSubPayload(const char* _pOrigDataSection,
       }
       break;
 
-    case SS_UNKNOWN:
+    case SubSection::unknown:
     default: {
-        std::string errMsg = XUtil::format("ERROR: Subsection '%s' not support by section '%s", _sSubSectionName.c_str(), getSectionKindAsString().c_str());
-        throw std::runtime_error(errMsg);
+        auto errMsg = boost::format("ERROR: Subsection '%s' not support by section '%s") % _sSubSectionName % getSectionKindAsString();
+        throw std::runtime_error(errMsg.str());
       }
       break;
   }
@@ -363,60 +395,62 @@ SectionSoftKernel::readSubPayload(const char* _pOrigDataSection,
 // -------------------------------------------------------------------------
 
 void
-SectionSoftKernel::writeObjImage(std::ostream& _oStream) const {
+SectionSoftKernel::writeObjImage(std::ostream& _oStream) const
+{
   XUtil::TRACE("SectionSoftKernel::writeObjImage");
 
   // Overlay the structure
   // Do we have enough room to overlay the header structure
   if (m_bufferSize < sizeof(soft_kernel)) {
-    std::string errMsg = XUtil::format("ERROR: Segment size (%d) is smaller than the size of the bmc structure (%d)", m_bufferSize, sizeof(soft_kernel));
-    throw std::runtime_error(errMsg);
+    auto errMsg = boost::format("ERROR: Segment size (%d) is smaller than the size of the bmc structure (%d)") % m_bufferSize % sizeof(soft_kernel);
+    throw std::runtime_error(errMsg.str());
   }
 
   // No look at the data
-  soft_kernel* pHdr = reinterpret_cast<soft_kernel *>(m_pBuffer);
+  auto pHdr = reinterpret_cast<soft_kernel*>(m_pBuffer);
 
-  const char* pFWBuffer = reinterpret_cast<const char *>(pHdr) + pHdr->m_image_offset;
+  auto pFWBuffer = reinterpret_cast<const char*>(pHdr) + pHdr->m_image_offset;
   _oStream.write(pFWBuffer, pHdr->m_image_size);
 }
 
 // -------------------------------------------------------------------------
 
 void
-SectionSoftKernel::writeMetadata(std::ostream& _oStream) const {
+SectionSoftKernel::writeMetadata(std::ostream& _oStream) const
+{
   XUtil::TRACE("SOFTKERNEL-METADATA");
 
   // Overlay the structure
   // Do we have enough room to overlay the header structure
   if (m_bufferSize < sizeof(soft_kernel)) {
-    std::string errMsg = XUtil::format("ERROR: Segment size (%d) is smaller than the size of the softkernel structure (%d)", m_bufferSize, sizeof(soft_kernel));
-    throw std::runtime_error(errMsg);
+    auto errMsg = boost::format("ERROR: Segment size (%d) is smaller than the size of the softkernel structure (%d)") % m_bufferSize % sizeof(soft_kernel);
+    throw std::runtime_error(errMsg.str());
   }
 
-  soft_kernel* pHdr = reinterpret_cast<soft_kernel *>(m_pBuffer);
+  auto pHdr = reinterpret_cast<soft_kernel*>(m_pBuffer);
 
-  XUtil::TRACE(XUtil::format("Original: \n"
+  XUtil::TRACE(boost::format("Original: \n"
                              "  mpo_name (0x%lx): '%s'\n"
                              "  m_image_offset: 0x%lx, m_image_size: 0x%lx\n"
                              "  mpo_version (0x%lx): '%s'\n"
                              "  mpo_md5_value (0x%lx): '%s'\n"
                              "  mpo_symbol_name (0x%lx): '%s'\n"
-                             "  m_num_instances: %d",
-                             pHdr->mpo_name, reinterpret_cast<char *>(pHdr) + pHdr->mpo_name,
-                             pHdr->m_image_offset, pHdr->m_image_size,
-                             pHdr->mpo_version, reinterpret_cast<char *>(pHdr) + pHdr->mpo_version,
-                             pHdr->mpo_md5_value, reinterpret_cast<char *>(pHdr) + pHdr->mpo_md5_value,
-                             pHdr->mpo_symbol_name, reinterpret_cast<char *>(pHdr) + pHdr->mpo_symbol_name,
-                             pHdr->m_num_instances));
+                             "  m_num_instances: %d")
+               % pHdr->mpo_name % (reinterpret_cast<char*>(pHdr) + pHdr->mpo_name)
+               % pHdr->m_image_offset % pHdr->m_image_size
+               % pHdr->mpo_version % (reinterpret_cast<char*>(pHdr) + pHdr->mpo_version)
+               % pHdr->mpo_md5_value % (reinterpret_cast<char*>(pHdr) + pHdr->mpo_md5_value)
+               % pHdr->mpo_symbol_name % (reinterpret_cast<char*>(pHdr) + pHdr->mpo_symbol_name)
+               % pHdr->m_num_instances);
 
   // Convert the data from the binary format to JSON
   boost::property_tree::ptree ptSoftKernel;
 
-  ptSoftKernel.put("mpo_name", reinterpret_cast<char *>(pHdr) + pHdr->mpo_name);
-  ptSoftKernel.put("mpo_version", reinterpret_cast<char *>(pHdr) + pHdr->mpo_version);
-  ptSoftKernel.put("mpo_md5_value", reinterpret_cast<char *>(pHdr) + pHdr->mpo_md5_value);
-  ptSoftKernel.put("mpo_symbol_name", reinterpret_cast<char *>(pHdr) + pHdr->mpo_symbol_name);
-  ptSoftKernel.put("m_num_instances", XUtil::format("%d", pHdr->m_num_instances).c_str());
+  ptSoftKernel.put("mpo_name", reinterpret_cast<char*>(pHdr) + pHdr->mpo_name);
+  ptSoftKernel.put("mpo_version", reinterpret_cast<char*>(pHdr) + pHdr->mpo_version);
+  ptSoftKernel.put("mpo_md5_value", reinterpret_cast<char*>(pHdr) + pHdr->mpo_md5_value);
+  ptSoftKernel.put("mpo_symbol_name", reinterpret_cast<char*>(pHdr) + pHdr->mpo_symbol_name);
+  ptSoftKernel.put("m_num_instances", (boost::format("%d") % pHdr->m_num_instances).str());
 
   boost::property_tree::ptree root;
   root.put_child("soft_kernel_metadata", ptSoftKernel);
@@ -429,7 +463,8 @@ SectionSoftKernel::writeMetadata(std::ostream& _oStream) const {
 void
 SectionSoftKernel::writeSubPayload(const std::string& _sSubSectionName,
                                    FormatType _eFormatType,
-                                   std::fstream&  _oStream) const {
+                                   std::fstream&  _oStream) const
+{
   // Some basic DRC checks
   if (m_pBuffer == nullptr) {
     std::string errMsg = "ERROR: Soft Kernel section does not exist.";
@@ -439,9 +474,9 @@ SectionSoftKernel::writeSubPayload(const std::string& _sSubSectionName,
   SubSection eSubSection = getSubSectionEnum(_sSubSectionName);
 
   switch (eSubSection) {
-    case SS_OBJ:
+    case SubSection::obj:
       // Some basic DRC checks
-      if (_eFormatType != Section::FT_RAW) {
+      if (_eFormatType != Section::FormatType::raw) {
         std::string errMsg = "ERROR: SOFT_KERNEL-OBJ only supports the RAW format.";
         throw std::runtime_error(errMsg);
       }
@@ -449,8 +484,8 @@ SectionSoftKernel::writeSubPayload(const std::string& _sSubSectionName,
       writeObjImage(_oStream);
       break;
 
-    case SS_METADATA: {
-        if (_eFormatType != Section::FT_JSON) {
+    case SubSection::metadata: {
+        if (_eFormatType != Section::FormatType::json) {
           std::string errMsg = "ERROR: SOFT_KERNEL-METADATA only supports the JSON format.";
           throw std::runtime_error(errMsg);
         }
@@ -459,17 +494,18 @@ SectionSoftKernel::writeSubPayload(const std::string& _sSubSectionName,
       }
       break;
 
-    case SS_UNKNOWN:
+    case SubSection::unknown:
     default: {
-        std::string errMsg = XUtil::format("ERROR: Subsection '%s' not support by section '%s", _sSubSectionName.c_str(), getSectionKindAsString().c_str());
-        throw std::runtime_error(errMsg);
+        auto errMsg = boost::format("ERROR: Subsection '%s' not support by section '%s") % _sSubSectionName % getSectionKindAsString();
+        throw std::runtime_error(errMsg.str());
       }
       break;
   }
 }
 
 void
-SectionSoftKernel::readXclBinBinary(std::fstream& _istream, const axlf_section_header& _sectionHeader) {
+SectionSoftKernel::readXclBinBinary(std::istream& _istream, const axlf_section_header& _sectionHeader)
+{
   Section::readXclBinBinary(_istream, _sectionHeader);
 
   // Extract the binary data as a JSON string
@@ -489,7 +525,7 @@ SectionSoftKernel::readXclBinBinary(std::fstream& _istream, const axlf_section_h
   boost::property_tree::ptree& ptSoftKernel = pt.get_child("soft_kernel_metadata");
 
   XUtil::TRACE_PrintTree("Current SOFT_KERNEL contents", pt);
-  std::string sName = ptSoftKernel.get<std::string>("mpo_name"); 
+  std::string sName = ptSoftKernel.get<std::string>("mpo_name");
 
   Section::m_sIndexName = sName;
 }
