@@ -54,6 +54,7 @@
 namespace xdp {
 using severity_level = xrt_core::message::severity_level;
 bool AieTracePluginUnified::live = false;
+bool AieTracePluginUnified::configuredOnePartition = false;
 
 AieTracePluginUnified::AieTracePluginUnified() : XDPPlugin() {
   AieTracePluginUnified::live = true;
@@ -100,7 +101,14 @@ void AieTracePluginUnified::updateAIEDevice(void *handle, bool hw_context_flow)
   if (!handle)
     return;
   
-  if (!((db->getStaticInfo()).continueXDPConfig(hw_context_flow))) {
+  if (!((db->getStaticInfo()).continueXDPConfig(hw_context_flow)))
+    return;
+
+  // In a multipartition scenario, if the user wants to trace one specific partition
+  // and we have already configured one partition, we can skip the rest of them
+  if ((xrt_core::config::get_aie_trace_settings_config_one_partition()) && (configuredOnePartition)) {
+    xrt_core::message::send(severity_level::warning, "XRT",
+      "AIE Trace: A previous partition has already been configured. Skipping current partition due to 'config_one_partition=true' setting.");
     return;
   }
   
@@ -156,6 +164,10 @@ void AieTracePluginUnified::updateAIEDevice(void *handle, bool hw_context_flow)
     return;
   }
   AIEData.valid = true; // initialize struct
+  
+  // If there are tiles configured for this xclbin, then we have configured the first matching xclbin and will not configure any upcoming ones
+  if ((xrt_core::config::get_aie_trace_settings_config_one_partition()) && !(AIEData.metadata->configMetricsEmpty()))
+    configuredOnePartition = true;
 
 #ifdef XDP_CLIENT_BUILD
   xrt::hw_context context = xrt_core::hw_context_int::create_hw_context_from_implementation(handle);
@@ -365,13 +377,14 @@ void AieTracePluginUnified::finishFlushAIEDevice(void *handle) {
   if (!handle)
     return;
 
-  auto itr = handleToAIEData.find(handle);
+  // mark the hw_ctx handle as invalid for current plugin  
+  (db->getStaticInfo()).unregisterPluginFromHwContext(handle);
 
+  auto itr = handleToAIEData.find(handle);
   if (itr == handleToAIEData.end())
     return;
 
   auto &AIEData = itr->second;
-
   if (!AIEData.valid)
     return;
 
@@ -383,7 +396,6 @@ void AieTracePluginUnified::finishFlushAIEDevice(void *handle) {
   if (AIEData.offloadManager)
     AIEData.offloadManager->flushAll(true);
 
-
   XDPPlugin::endWrite();
 
   handleToAIEData.erase(itr);
