@@ -18,8 +18,7 @@
 #include "xocl/config.h"
 #include "xocl/core/error.h"
 #include "detail/device.h"
-#include "plugin/xdp/profile.h"
-#include "plugin/xdp/lop.h"
+#include "plugin/xdp/profile_v2.h"
 
 namespace xocl {
 
