@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2016-2017 Xilinx, Inc
+ * Copyright (C) 2016-2020 Xilinx, Inc
  *
  * Licensed under the Apache License, Version 2.0 (the "License"). You may
  * not use this file except in compliance with the License. A copy of the
@@ -17,6 +17,12 @@
 #ifndef xrtx_uuid_h_
 #define xrtx_uuid_h_
 
-#include "core/include/experimental/xrt_uuid.h"
+#include "core/include/xrt/xrt_uuid.h"
+
+namespace xrt_xocl {
+
+using xrt::uuid;
+
+}
 
 #endif
